#!/usr/bin/env python
'''
################################################################################
#
# This is from hoc to Python for the simplistic LFP to biophysical LFP model
################################################################################
'''
# importing some modules, setting some matplotlib values for pl.plot.
import LFPy
import numpy as np
import nrn
import scipy.io
from neuron import h
from itertools import chain
import matplotlib.pyplot as plt
import os
import sys
import string
import urllib2
import zipfile
from matplotlib.collections import PolyCollection
plt.rcParams.update({'font.size' : 12,
                     'figure.facecolor' : '1',
                     'figure.subplot.wspace' : 0.5,
                     'figure.subplot.hspace' : 0.5})

#plt.interactive(1)
#plt.close('all')

#seed for random generation
np.random.seed(1234)

#h('load_file("fig-5a.hoc")')


#########################################
##### SOM/BSC connection parameters
csb_str=sys.argv[1]     #string
#c_sb=decimal.Decimal('csb_str')
#Decimal(csb_str)
c_sb=float(csb_str)
c_bs=0.64*c_sb
cbs_str=str(c_bs)

gsb_str=sys.argv[2]
gbs_str=sys.argv[3]
print gsb_str
print gbs_str
gsb_raw=float(gsb_str)
gbs_raw=float(gbs_str)
g_sb=gsb_raw#*nS
g_bs=gbs_raw#*nS


################################################################################
# Main simulation procedure
################################################################################

h('csb='+ str(c_sb))
h('gsb='+ str(g_sb))
h('gbs='+ str(g_bs))


################################################################################
# Main simulation procedure
################################################################################

#define cell parameters used as input to cell-class
cellParameters = {
    'morphology' : 'fig-5a_basal_exc.hoc',
#    'rm' : 20000,               # membrane resistance
#    'cm' : 1.9 ,                 # membrane capacitance
#    'Ra' : 80,                 # axial resistance
    'v_init' : -70,             # initial crossmembrane potential
    'e_pas' : -70,              # reversal potential passive mechs
    'passive' : True,           # switch on passive mechs
    'nsegs_method' : 'lambda_f',# method for setting number of segments,
    'lambda_f' : 100,           # segments are isopotential at this frequency
    'timeres_NEURON' : 2**-4,   # dt of LFP and NEURON simulation.
    'timeres_python' : 2**-4,
    'tstartms' : -100,          #start time, recorders start at t=0
    'tstopms' : 5000,            #stop time of simulation
    #'custom_code'  : ['active_declarations_example3.hoc'], # will run this file
}

# Create cell
cell = LFPy.Cell(**cellParameters)
# Align cell
cell.set_rotation(x=20, y=0, z=-15.5)

###################################################
# Create a grid of measurement locations, in (mum)
X, Z = np.mgrid[-500:501:10, -300:801:5] #np.mgrid[-300:301:10, -200:701:5]
Y = np.zeros(X.shape)

# Define electrode parameters
grid_electrode_parameters = {
    'sigma' : 0.3,      # extracellular conductivity
    'x' : X.flatten(),  # electrode requires 1d vector of positions
    'y' : Y.flatten(),
    'z' : Z.flatten()
}
##################################################

# Define electrode geometry corresponding to a laminar electrode, where contact
# points have a radius r, surface normal vectors N, and LFP calculated as the
# average LFP in n random points on each contact:
N = np.empty((16, 3))
for i in range(N.shape[0]): N[i,] = [1, 0, 0] #normal unit vec. to contacts

# put parameters in dictionary
point_electrode_parameters = {
    'sigma' : 0.3,              # Extracellular potential
    'x' : np.zeros(2) ,      # x,y,z-coordinates of electrode contacts
    'y' : np.zeros(2),
    'z' : np.linspace(150, 250, 2),# standard is -100,500,6
    'n' : 20,
    'r' : 10,
    'N' : N,
}

# Parameters for the cell.simulate() call, recording membrane- and syn.-currents
simulationParameters = {
    'rec_imem' : True,  # Record Membrane currents during simulation
    'rec_isyn' : True,  # Record synaptic currents
}


#######################################################



#Initialize cell instance, using the LFPy.Cell class
#cell = LFPy.Cell(**cellParameters)

# perform NEURON simulation, results saved as attributes in the cell instance
# Initialize electrode geometry, then calculate the LFP, using the
# LFPy.RecExtElectrode class. Note that now cell is given as input to electrode
# and created after the NEURON simulations are finished


print('simulating LFPs....')
cell.simulate(**simulationParameters)


# Create electrode objects
grid_electrode = LFPy.RecExtElectrode(cell,**grid_electrode_parameters)
point_electrode = LFPy.RecExtElectrode(cell, **point_electrode_parameters)

# Calculate LFPs
grid_electrode.calc_lfp()
point_electrode.calc_lfp()

print('done')

#plotting some variables and geometry, saving output to .pdf.
#from example_sup import plot_ex3
#fig = plot_ex3(cell, point_electrode)
#fig.savefig('example7testfig5.pdf', dpi=300)
#plt.show()


#create a plot
#fig = prepfig(1, 8.5, 6.)
fig = plt.figure()

ax = fig.add_axes([.4,.1,.55,.8], aspect='equal', frameon=False)
cax = fig.add_axes([0.85, 0.4, 0.02, 0.4], aspect='equal', frameon=False) #([0.85, 0.4, 0.01, 0.2], frameon=False)

LFP = np.max(np.abs(grid_electrode.LFP),1).reshape(X.shape)
im = ax.contour(X, Z, np.log10(LFP), 
            50,
           #V=np.arange(-25.,-10.,0.1),
           cmap='rainbow',
           #linewidths=2,
           zorder=-2)
#cbar = fig.colorbar(im, fraction=0.1, shrink=0.4)
cbar = fig.colorbar(im, cax=cax)
cbar.set_label('$|\Phi(\mathbf{r}, t)|_\mathrm{max}$ (nV)')
cbar.outline.set_visible(False)
#get some log-linear tickmarks and ticklabels
ticks = np.arange(np.ceil(np.log10(LFP.min())), np.floor(np.log10(LFP.max())))
cbar.set_ticks(ticks)
cbar.set_ticklabels(10.**ticks * 1E6) #mV -> nV

#plot morphology
zips = []
for x, z in cell.get_idx_polygons():
    zips.append(zip(x, z))
polycol = PolyCollection(zips,
                         edgecolors='none',
                         facecolors='k')
ax.add_collection(polycol)

ax.plot([100, 200], [-400, -400], 'k', lw=1, clip_on=False)
ax.text(150, -470, r'100$\mu$m', va='center', ha='center')

ax.axis('off')


ax.plot(cell.xmid[cell.synidx],cell.zmid[cell.synidx], 'o', ms=5,
        markeredgecolor='k',
        markerfacecolor='r')

color_vec = ['blue','green']
for i in xrange(2):
    ax.plot(point_electrode_parameters['x'][i],
                    point_electrode_parameters['z'][i],'o',ms=6,
                    markeredgecolor='none',
                    markerfacecolor=color_vec[i])

plt.axes([.15,.10,.25,.25])
plt.plot(cell.tvec,point_electrode.LFP[0]*1e6,color=color_vec[0], clip_on=False)
plt.ylabel(r'$\Phi_2(\mathbf{r},t)$ (nV)') #rotation='horizontal')
plt.xlabel(r'$t$ (ms)')
plt.axis('tight')
ax = plt.gca()
for loc, spine in ax.spines.iteritems():
    if loc in ['right', 'top']:
        spine.set_color('none')            
ax.xaxis.set_ticks_position('bottom')
ax.yaxis.set_ticks_position('left')


plt.axes([.15,.375,.25,.25])
plt.plot(cell.tvec,point_electrode.LFP[1]*1e6,color=color_vec[1], clip_on=False)
plt.ylabel(r'$\Phi_1(\mathbf{r}, t)$ (nV)') #rotation='horizontal')
plt.axis('tight')
ax = plt.gca()
for loc, spine in ax.spines.iteritems():
    if loc in ['right', 'top']:
        spine.set_color('none')            
ax.xaxis.set_ticks_position('bottom')
ax.yaxis.set_ticks_position('left')
ax.set_xticklabels([])

#####################commenting out the synapse input plot
#plt.axes([.15,.65,.25,.25])
#plt.plot(cell.tvec,synapse.i*1E3, color='red', clip_on=False)
#plt.ylabel(r'$i_\mathrm{syn}(t)$ (pA)') #, rotation='horizontal')
#plt.axis('tight')
#ax = plt.gca()
#for loc, spine in ax.spines.iteritems():
#    if loc in ['right', 'top']:
#        spine.set_color('none')            
#ax.xaxis.set_ticks_position('bottom')
#ax.yaxis.set_ticks_position('left')
#ax.set_xticklabels([])
############################################


# fig = plot_ex1(cell, electrode, X, Y, Z)
# Optionally save figure (uncomment the line below)
plt.savefig('exampletestfig5_gridlike_ex1.pdf', dpi=300)
